﻿using System;
using Assima.Infrastructure.AsyncToSyncCorrelator;
using Assima.Infrastructure.Policy.Intranet.Server;
using Assima.Infrastructure.Simulator.Contract.DataContract;
using Assima.Infrastructure.Simulator.Intranet;
using Assima.Infrastructure.Simulator.Proxy.Queued;
using Assima.Infrastructure.Simulator.Helpers;
using Newtonsoft.Json;
using ServiceModelEx;
using System.Linq;
using Assima.ResourceAccess.LLBLGen.SelfService.CollectionClasses;
using Assima.ResourceAccess.LLBLGen.Adapter.EntityClasses;
using Assima.ResourceAccess.LLBLGen.Adapter.HelperClasses;
using Assima.ResourceAccess.LLBLGen.Adapter.DatabaseSpecific;
using Assima.ResourceAccess.LLBLGen.Adapter.Linq;
using System.Data.SqlClient;
using System.Configuration;
using System.Reflection;
using NHibernate.Cfg;
using NHibernate.Driver;
using NHibernate.Dialect;
using NHibernate.Linq;
using Assima.ResourceAccess.NHibernateDB;
using System.Data;
using Dapper;
using Assima.ResourceAccess.LLBLGen.SelfService.DaoClasses;
using SD.LLBLGen.Pro.ORMSupportClasses;
using Assima.ResourceAccess.EntityFramework;
using FluentAssertions;
using System.Collections.Generic;

namespace Assima.Service.Infrastructure.Simulator.Intranet
{
    [IntranetPolicy]
    public class IntranetServiceManager : AsyncToSyncCorrelator, IIntranetServiceContract
    {
        #region sql

        public const string ComplexSQL = @"SELECT DISTINCT 
	[UnionAll1].[LocalizedItemID]
        AS[LocalizedItemID], 
	[UnionAll1].[LocalizedTargetId]
        AS[LocalizedTargetID], 
	[UnionAll1].[LocalizedTargetTypeID]
        AS[LocalizedTargetTypeID]
INTO #LocalizedShortcutTable 
FROM(
    SELECT
        [Shortcut].[LocalizedItemID] AS[LocalizedItemID],
        [Shortcut].[LocalizedTargetId] AS[LocalizedTargetId],
        [Shortcut].[LocalizedTargetTypeID] AS[LocalizedTargetTypeID],
        [Shortcut].[LocalizedTargetId] AS [LocalizedTargetId1]
    FROM [dbo].[ContentItems_LocalizedShortcut] AS [Shortcut]
    WHERE [Shortcut].[LocalizedItemID] IN (
        SELECT[n]
        FROM @PublishedItemMetadataIds)
        UNION ALL
        SELECT
            [Join2].[LocalizedItemID1]
        AS[LocalizedItemID], 
			[Join2].[LocalizedTargetId]
        AS[LocalizedTargetId], 
			[Join2].[LocalizedTargetTypeID]
        AS[LocalizedTargetTypeID], 
			[Join2].[LocalizedTargetId]
        AS[LocalizedTargetId1]
FROM[dbo].[ContentItems_Published]
        AS[PublishedItem]
INNER JOIN(
SELECT
  [ContentItems_LocalizedShortcut].[LocalizedItemID] AS[LocalizedItemID1],
  [ContentItems_LocalizedShortcut].[LocalizedTargetId] AS[LocalizedTargetId],
  [ContentItems_LocalizedShortcut].[LocalizedTargetTypeID] AS[LocalizedTargetTypeID],
  [Join1].[ContentItems_LocalizedItem_LocalizedItemID], [Join1].[ContentItems_Published_LocalizedItemID]
            FROM  [dbo].[ContentItems_LocalizedShortcut] AS [ContentItems_LocalizedShortcut]
INNER JOIN  (
  SELECT
      [ContentItems_LocalizedItem].[LocalizedItemID] AS[ContentItems_LocalizedItem_LocalizedItemID],
      [ContentItems_Published].[LocalizedItemID] AS [ContentItems_Published_LocalizedItemID]
  FROM  [dbo].[ContentItems_LocalizedItem] AS [ContentItems_LocalizedItem]
  INNER JOIN[dbo].[ContentItems_Published] AS [ContentItems_Published]
  ON [ContentItems_LocalizedItem].[LocalizedItemID] = [ContentItems_Published].[LocalizedItemID]) AS[Join1]
 ON[Join1].[ContentItems_LocalizedItem_LocalizedItemID] = [ContentItems_LocalizedShortcut].[LocalizedTargetId] ) AS[Join2]
ON[PublishedItem].[LocalizedItemID] = [Join2].[ContentItems_Published_LocalizedItemID]

WHERE([PublishedItem].[LocalizedContentItemId] IN (
                    SELECT[n] FROM @PublishedItemMetadataIds)) 
                        OR([PublishedItem].[LocalizedItemID] IN (SELECT[n] FROM @PublishedItemMetadataIds))) 
						AS[UnionAll1]

SELECT[LocalizedItem].[LocalizedItemID]
        AS[LocalizedItemId], 
	[LocalizedItem].[LocalizedItemID]
        AS[LocalizedTargetId], 
	[LocalizedItem].[LanguageID]
        AS[LanguageId], 
	[LocalizedItem].[Name]
        AS[Name], 
	[LocalizedItem].[PublishState]
        AS[PublishState], 
	[LocalizedItem].[LocalizedType]
        AS[PublishedItemTypeId], 
	[PublishedItem].[SourceTypeID]
        AS[SourceTypeId]
FROM[dbo].[ContentItems_LocalizedItem]
        AS[LocalizedItem] LEFT OUTER JOIN[dbo].[ContentItems_Published]
        AS[PublishedItem]
ON[LocalizedItem].[LocalizedItemID] = [PublishedItem].[LocalizedItemID]
        WHERE[LocalizedItem].[LocalizedItemID] IN(SELECT[LocalizedTargetID] FROM #LocalizedShortcutTable)";

        #endregion

        #region ServiceCalls
        public void Oneway(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);
        }

        public void OnewayWithMsmqResponseService(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            ResponseServiceContext context = ResponseServiceContext.Current;
            if (context != null)
            {
                CalleeResponseInfo callee = context.CurrentResponseInfo;

                if (callee != null)
                {
                    using (QueuedResponseServiceClient client = new QueuedResponseServiceClient(new Uri(callee.ResponseServiceAddress), callee.ResponseId))
                    {
                        client.Respond(new ResponseDataContract(input.TestName, input.ClientRequestTime, serverReceived, ServiceHelper.GetResponseData()));
                    }
                }
            }
        }

        public void OnewayWithTcpResponseService(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            ResponseServiceContext context = ResponseServiceContext.Current;
            if (context != null)
            {
                CalleeResponseInfo callee = context.CurrentResponseInfo;

                if (callee != null)
                {
                    using (IntranetResponseServiceClient client = new IntranetResponseServiceClient(new Uri(callee.ResponseServiceAddress), callee.ResponseId))
                    {
                        client.Respond(new ResponseDataContract(input.TestName, input.ClientRequestTime, serverReceived, ServiceHelper.GetResponseData()));
                    }
                }
            }
        }

        public ResponseDataContract RequestResponse(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            return new ResponseDataContract(input.TestName, input.ClientRequestTime, serverReceived, ServiceHelper.GetResponseData());
        }


        public void Respond(RequestDataContract input)
        {
            ResponseServiceContext context = ResponseServiceContext.Current;
            if (context != null)
            {
                CalleeResponseInfo callee = context.CurrentResponseInfo;

                JsonSerializerSettings settings = new JsonSerializerSettings() { TypeNameHandling = TypeNameHandling.Auto };
                string json = JsonConvert.SerializeObject(input, settings);
                AsyncToSyncCorrelator.OnAsyncResponseReceived(callee.ResponseId, json);
            }
        }

        #endregion ServiceCalls

        #region DB Calls

        public ResponseDataContract RequestResponse_DBCall(DBRequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            switch(input.DBCall)
            {
                case DBCall.AdoNet:
                    ADONET(input);
                    break;
                case DBCall.Dapper:
                    Dapper(input);
                    break;
                case DBCall.LLBLGen_Self:
                    LLBLGen_Self(input);
                    break;
                case DBCall.LLBLGen_Adapter:
                    LLBLGen_Adapter(input);
                    break;
                case DBCall.LLBLGen_Linq:
                    LLBLGen_Linq(input);
                    break;
                case DBCall.NHibernate_Linq:
                    NHibernate_Linq(input);
                    break;
                case DBCall.NHibernate_Session:
                    NHibernate_Session(input);
                    break;
                case DBCall.EF:
                    EF(input);
                    break;
                case DBCall.MixedCalls:
                    MixedCalls(input);
                    break;
                default:
                    throw new Exception("Unknown Call");
            }

            return new ResponseDataContract(input.TestName, input.ClientRequestTime, serverReceived, "Completed");
        }

        #region Ado.Net

        private void ADONET(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_AdoDotNet_Single_Table(input);
                    break;
                case DBCallType.Join:
                    RequestResponse_AdoDotNet_Join(input);
                    break;
                case DBCallType.Insert:
                    RequestResponse_AdoDotNet_Insert(input);
                    break;
                case DBCallType.Complex:
                    RequestResponse_AdoDotNet_Complex(input);
                    break;
                case DBCallType.DoubleCall:
                    RequestResponse_AdoDotNet_DoubleCall(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_AdoDotNet_Single_Table(RequestDataContract input)
        {
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString()))
            {
                connection.Open();
                SqlTransaction tran = connection.BeginTransaction(System.Data.IsolationLevel.ReadCommitted);
                SqlCommand command = new SqlCommand(@"SELECT * FROM[dbo].[ContentItems_Published]", connection, tran);
                command.CommandType = System.Data.CommandType.Text;
                SqlDataReader reader = command.ExecuteReader();
                if (!reader.HasRows)
                    throw new Exception("No Rows Returned");
                reader.Close();
            }
        }
        public void RequestResponse_AdoDotNet_Join(RequestDataContract input)
        {
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString()))
            {
                connection.Open();
                SqlTransaction tran = connection.BeginTransaction(System.Data.IsolationLevel.ReadCommitted);
                SqlCommand command = new SqlCommand(@"SELECT * FROM
                                                    [DBO].[ContentItems_LocalizedItem] [ContentItems_LocalizedItem] INNER JOIN [DBO].[ContentItems_Published] [ContentItems_Published]
                                                    ON [ContentItems_LocalizedItem].LocalizedItemID = [ContentItems_Published].LocalizedItemID", connection, tran);
                command.CommandType = System.Data.CommandType.Text;
                SqlDataReader reader = command.ExecuteReader();
                if (!reader.HasRows)
                    throw new Exception("No Rows Returned");
                reader.Close();
                tran.Commit();
                connection.Close();
            }
        }

        public void RequestResponse_AdoDotNet_Insert(RequestDataContract input)
        {
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString()))
            {
                connection.Open();
                SqlTransaction tran = connection.BeginTransaction(System.Data.IsolationLevel.ReadCommitted);
                SqlCommand command = new SqlCommand(@"INSERT INTO [dbo].[Dump] ([text]) VALUES ('" + input.TestName + "');", connection, tran);
                command.CommandType = System.Data.CommandType.Text;
                int rows = command.ExecuteNonQuery();
                rows.Should().BeGreaterOrEqualTo(1);
                tran.Commit();
                connection.Close();
            }
        }

        public void RequestResponse_AdoDotNet_Complex(RequestDataContract input)
        {
            DataTable tableParameter = new DataTable();
            tableParameter.Columns.Add("n");
            tableParameter.Rows.Add(142);
            
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString()))
            {
                connection.Open();
                SqlTransaction tran = connection.BeginTransaction(System.Data.IsolationLevel.ReadCommitted);
                SqlCommand command = new SqlCommand(ComplexSQL, connection, tran);
                command.CommandType = System.Data.CommandType.Text;
                SqlParameter parameter = command.Parameters.AddWithValue("@PublishedItemMetadataIds", tableParameter);
                parameter.TypeName = "Ids";
                SqlDataReader reader = command.ExecuteReader();
                reader.HasRows.Should().BeTrue();
                reader.Close();
                tran.Commit();
                connection.Close();
            }
        }

        public void RequestResponse_AdoDotNet_DoubleCall(RequestDataContract input)
        {
            DataTable tableParameter = new DataTable();
            tableParameter.Columns.Add("n");
            tableParameter.Rows.Add(142);

            using (SqlConnection connection = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString()))
            {
                connection.Open();
                SqlTransaction tran = connection.BeginTransaction(System.Data.IsolationLevel.ReadCommitted);
                SqlCommand command = new SqlCommand(@"SELECT cli.*, cip.* FROM
                                                    [DBO].[ContentItems_LocalizedItem] cli INNER JOIN [DBO].[ContentItems_Published] cip
                                                    ON cli.LocalizedItemID = cip.LocalizedItemID", connection, tran);
                command.CommandType = System.Data.CommandType.Text;
                SqlDataReader reader = command.ExecuteReader();
                reader.HasRows.Should().BeTrue("items");
                reader.Close();
                tran.Commit();
                connection.Close();
            }
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString()))
            {
                connection.Open();
                SqlTransaction tran = connection.BeginTransaction(System.Data.IsolationLevel.ReadCommitted);
                SqlCommand command = new SqlCommand(@"SELECT s.*, l.* FROM 
                                                    [DBO].[ContentItems_LocalizedShortcut] s inner join [dbo].[ContentItems_LocalizedItem] l
                                                    ON s.LocalizedItemID = l.LocalizedItemID", connection, tran);
                command.CommandType = System.Data.CommandType.Text;
                SqlDataReader reader = command.ExecuteReader();
                reader.HasRows.Should().BeTrue("items2");
                reader.Close();
                tran.Commit();
                connection.Close();
            }
        }




        #endregion

        #region LLBLGen

        private void LLBLGen_Self(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_LLBLGen_Self_Single_Table(input);
                    break;
                case DBCallType.Join:
                    RequestResponse_LLBLGen_Self_Join(input);
                    break;
                case DBCallType.Insert:
                    RequestResponse_LLBLGen_Self_Insert(input);
                    break;
                case DBCallType.Complex:
                    RequestResponse_LLBLGen_Self_Complex(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        #region Self

        public void RequestResponse_LLBLGen_Self_Single_Table(RequestDataContract input)
        {
            ContentItemsPublishedCollection items = new ContentItemsPublishedCollection();
            items.GetMulti(null);
            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }

        public void RequestResponse_LLBLGen_Self_Join(RequestDataContract input)
        {
            var items = new ContentItemsLocalizedItemCollection();

            IRelationCollection relations = new RelationCollection(Assima.ResourceAccess.LLBLGen.SelfService.EntityClasses.ContentItemsLocalizedItemEntity.Relations.ContentItemsPublishedEntityUsingLocalizedItemId);

            items.GetMulti(null, relations);

            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }
    

        public void RequestResponse_LLBLGen_Self_Insert(RequestDataContract input)
        {
            Assima.ResourceAccess.LLBLGen.SelfService.EntityClasses.DumpEntity de = new Assima.ResourceAccess.LLBLGen.SelfService.EntityClasses.DumpEntity();
            de.Text = input.TestName;
            bool saved = de.Save();
            saved.Should().BeTrue();
        }

        public void RequestResponse_LLBLGen_Self_Complex(RequestDataContract input)
        {
            throw new NotImplementedException();
        }

        #endregion Self

        #region Adapter

        private void LLBLGen_Adapter(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_LLBLGen_Adapter_Single_Table(input);
                    break;
                case DBCallType.Join:
                    RequestResponse_LLBLGen_Adapter_Join(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_LLBLGen_Adapter_Single_Table(RequestDataContract input)
        {
            EntityCollection<ContentItemsPublishedEntity> items = new EntityCollection<ContentItemsPublishedEntity>();
            using (DataAccessAdapter adapter = new DataAccessAdapter())
            {
                adapter.FetchEntityCollection(items, null);
            }
            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }

        public void RequestResponse_LLBLGen_Adapter_Join(RequestDataContract input)
        {
            // Fields to fetch
            ResultsetFields fields = new ResultsetFields(26);
            fields.DefineField(ContentItemsPublishedFields.Bytes, 0);
            fields.DefineField(ContentItemsPublishedFields.ClassName, 1);
            fields.DefineField(ContentItemsPublishedFields.Extension, 2);
            fields.DefineField(ContentItemsPublishedFields.FileGuid, 3);
            fields.DefineField(ContentItemsPublishedFields.LocalizedContentItemId, 4);
            fields.DefineField(ContentItemsPublishedFields.LocalizedItemId, 5);
            fields.DefineField(ContentItemsPublishedFields.OpenInNewWindow, 6);
            fields.DefineField(ContentItemsPublishedFields.OutputFormat, 7);
            fields.DefineField(ContentItemsPublishedFields.PublicationTemplateId, 8);
            fields.DefineField(ContentItemsPublishedFields.PublishedVersionId, 9);
            fields.DefineField(ContentItemsPublishedFields.SourceTypeId, 10);
            fields.DefineField(ContentItemsPublishedFields.SourceVersionId, 11);
            fields.DefineField(ContentItemsLocalizedItemFields.CreationDate, 12);
            fields.DefineField(ContentItemsLocalizedItemFields.CreatorId, 13);
            fields.DefineField(ContentItemsLocalizedItemFields.Description, 14);
            fields.DefineField(ContentItemsLocalizedItemFields.HasPicture, 15);
            fields.DefineField(ContentItemsLocalizedItemFields.IsLockedBy, 16);
            fields.DefineField(ContentItemsLocalizedItemFields.ItemId, 17);
            fields.DefineField(ContentItemsLocalizedItemFields.LanguageId, 18);
            fields.DefineField(ContentItemsLocalizedItemFields.LastModificationBy, 19);
            fields.DefineField(ContentItemsLocalizedItemFields.LastModificationDate, 20);
            fields.DefineField(ContentItemsLocalizedItemFields.LocalizationStrategy, 21);
            fields.DefineField(ContentItemsLocalizedItemFields.LocalizedType, 22);
            fields.DefineField(ContentItemsLocalizedItemFields.Name, 23);
            fields.DefineField(ContentItemsLocalizedItemFields.PublishState, 24);
            fields.DefineField(ContentItemsLocalizedItemFields.UniqueIdentifier, 25);

            IRelationPredicateBucket bucket = new RelationPredicateBucket();
            bucket.Relations.Add(ContentItemsPublishedEntity.Relations.ContentItemsLocalizedItemEntityUsingLocalizedContentItemId);

            //IRelationCollection releations = new RelationCollection();

            DataTable dynamicList = new DataTable();
            using (DataAccessAdapter adapter = new DataAccessAdapter())
            {
                adapter.FetchTypedList(fields, dynamicList, bucket, 0, null, true);
            }
            if (fields == null)
                throw new Exception("No Rows Returned");
        }

        #endregion Adapter

        #region Linq

        private void LLBLGen_Linq(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_LLBLGen_Linq_Single_Table(input);
                    break;
                case DBCallType.Join:
                    RequestResponse_LLBLGen_Linq_Join(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_LLBLGen_Linq_Single_Table(RequestDataContract input)
        {
            using (DataAccessAdapter adapter = new DataAccessAdapter())
            {
                LinqMetaData metaData = new LinqMetaData();
                metaData.AdapterToUse = adapter;

                var items = (from p in metaData.ContentItemsPublished
                             select p).ToList();
                if (items == null || !items.Any())
                    throw new Exception("No Rows Returned");
            }
        }

             public void RequestResponse_LLBLGen_Linq_Join(RequestDataContract input)
        {
            using (DataAccessAdapter adapter = new DataAccessAdapter())
            {
                LinqMetaData metaData = new LinqMetaData();
                metaData.AdapterToUse = adapter;

                var items = (from p in metaData.ContentItemsPublished
                             join c in metaData.ContentItemsLocalizedItem on p.LocalizedItemId equals c.LocalizedItemId
                             select new
                             {
                                 p.Bytes,
                                 p.ClassName,
                                 p.Extension,
                                 p.FileGuid,
                                 p.LocalizedContentItemId,
                                 p.LocalizedItemId,
                                 p.OpenInNewWindow,
                                 p.PublicationTemplateId,
                                 p.PublishedVersionId,
                                 p.SourceTypeId,
                                 p.SourceVersionId,
                                 c.CreationDate,
                                 c.CreatorId,
                                 c.Description,
                                 c.HasPicture,
                                 c.IsLockedBy,
                                 c.ItemId,
                                 c.LanguageId,
                                 c.LastModificationBy,
                                 c.LastModificationDate,
                                 c.LocalizationStrategy,
                                 c.LocalizedType,
                                 c.Name,
                                 c.PublishState,
                                 c.UniqueIdentifier

                             }).ToList();
                if (items == null || !items.Any())
                    throw new Exception("No Rows Returned");
            }
        }

        #endregion Linq

        #endregion LLBLGen

        #region EF

        private void EF(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_EF_Single_Table(input);
                    break;
                case DBCallType.Join:
                    RequestResponse_EF_Join(input);
                    break;
                case DBCallType.Complex:
                    RequestResponse_EF_Complex(input);
                    break;
                case DBCallType.DoubleCall:
                    RequestResponse_EF_DoubleCall(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_EF_Single_Table(RequestDataContract input)
        {
            VimagoEntities context = new VimagoEntities();
            var items = (from c in context.ContentItems_Published
                    select c).ToList();
            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }

        public void RequestResponse_EF_Join(RequestDataContract input)
        {
            VimagoEntities context = new VimagoEntities();
            var items = (from p in context.ContentItems_Published
                         join c in context.ContentItems_LocalizedItem on p.LocalizedItemID equals c.LocalizedItemID
                         select new
                         {
                             p.Bytes,
                             p.ClassName,
                             p.Extension,
                             p.FileGUID,
                             p.LocalizedContentItemId,
                             p.LocalizedItemID,
                             p.OpenInNewWindow,
                             p.PublicationTemplateId,
                             p.PublishedVersionId,
                             p.SourceTypeID,
                             p.SourceVersionId,
                             c.CreationDate,
                             c.CreatorID,
                             c.Description,
                             c.HasPicture,
                             c.IsLockedBy,
                             c.ItemID,
                             c.LanguageID,
                             c.LastModificationBy,
                             c.LastModificationDate,
                             c.LocalizationStrategy,
                             c.LocalizedType,
                             c.Name,
                             c.PublishState,
                             c.UniqueIdentifier

                         }).ToList();
            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }

        public void RequestResponse_EF_Complex(RequestDataContract input)
        {
            throw new NotImplementedException();
            //VimagoEntities context = new VimagoEntities();
            //var items = (from p in context.ContentItems_Published
            //             join c in context.ContentItems_LocalizedItem on p.LocalizedItemID equals c.LocalizedItemID
            //             select new
            //             {
            //                 p.Bytes,
            //                 p.ClassName,
            //                 p.Extension,
            //                 p.FileGUID,
            //                 p.LocalizedContentItemId,
            //                 p.LocalizedItemID,
            //                 p.OpenInNewWindow,
            //                 p.PublicationTemplateId,
            //                 p.PublishedVersionId,
            //                 p.SourceTypeID,
            //                 p.SourceVersionId,
            //                 c.CreationDate,
            //                 c.CreatorID,
            //                 c.Description,
            //                 c.HasPicture,
            //                 c.IsLockedBy,
            //                 c.ItemID,
            //                 c.LanguageID,
            //                 c.LastModificationBy,
            //                 c.LastModificationDate,
            //                 c.LocalizationStrategy,
            //                 c.LocalizedType,
            //                 c.Name,
            //                 c.PublishState,
            //                 c.UniqueIdentifier

            //             }).ToList();
            //if (items == null || !items.Any())
            //    throw new Exception("No Rows Returned");
        }

        public void RequestResponse_EF_DoubleCall(RequestDataContract input)
        {
            using (VimagoEntities context = new VimagoEntities())
            {
                var items = (from p in context.ContentItems_Published
                             join c in context.ContentItems_LocalizedItem on p.LocalizedItemID equals c.LocalizedItemID
                             select new
                             {
                                 p.Bytes,
                                 p.ClassName,
                                 p.Extension,
                                 p.FileGUID,
                                 p.LocalizedContentItemId,
                                 p.LocalizedItemID,
                                 p.OpenInNewWindow,
                                 p.PublicationTemplateId,
                                 p.PublishedVersionId,
                                 p.SourceTypeID,
                                 p.SourceVersionId,
                                 c.CreationDate,
                                 c.CreatorID,
                                 c.Description,
                                 c.HasPicture,
                                 c.IsLockedBy,
                                 c.ItemID,
                                 c.LanguageID,
                                 c.LastModificationBy,
                                 c.LastModificationDate,
                                 c.LocalizationStrategy,
                                 c.LocalizedType,
                                 c.Name,
                                 c.PublishState,
                                 c.UniqueIdentifier

                             }).ToList();

                var items2 = (from l in context.ContentItems_LocalizedShortcut
                              join c in context.ContentItems_LocalizedItem on l.LocalizedItemID equals c.LocalizedItemID
                              select new
                              {
                                  l.LocalizedItemID,
                                  l.LocalizedTargetId,
                                  l.LocalizedTargetTypeID,
                                  c.CreationDate,
                                  c.CreatorID,
                                  c.Description,
                                  c.HasPicture,
                                  c.IsLockedBy,
                                  c.ItemID,
                                  c.LanguageID,
                                  c.LastModificationBy,
                                  c.LastModificationDate,
                                  c.LocalizationStrategy,
                                  c.LocalizedType,
                                  c.Name,
                                  c.PublishState,
                                  c.UniqueIdentifier

                              }).ToList();
                items.Should().NotBeNull("items");
                items.Any().Should().BeTrue("items");
                items2.Should().NotBeNull("items2");
                items2.Any().Should().BeTrue("items2");
            }
        }

        #endregion EF

        #region NHibernate

        #region Linq

        private void NHibernate_Linq(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_NHibernate_Linq_Single_Table(input);
                    break;
                case DBCallType.Join:
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_NHibernate_Linq_Single_Table(RequestDataContract input)
        {
            string conn = ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString();
            var cfg = new NHibernate.Cfg.Configuration();
            cfg.DataBaseIntegration(x =>
            {
                x.ConnectionString = conn;
                x.Driver<SqlClientDriver>();
                x.Dialect<MsSql2012Dialect>();
            });

            cfg.AddAssembly(Assembly.GetAssembly(typeof(ResourceAccess.NHibernateDB.ContentItems_Published)));
            var sessionFactory = cfg.BuildSessionFactory();
            using (var session = sessionFactory.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var items = (from p in session.Query<ResourceAccess.NHibernateDB.ContentItems_Published>()
                                    select p).ToList();
                        //session.CreateCriteria<ContentItems_Published>().List<ContentItems_Published>();
                    tx.Commit();
                    if (items == null || !items.Any())
                        throw new Exception("No Rows Returned");
                }
            }
        }

        #endregion Linq

        #region Session

        private void NHibernate_Session(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_NHibernate_Session_Single_Table(input);
                    break;
                case DBCallType.Join:
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_NHibernate_Session_Single_Table(RequestDataContract input)
        {
            string conn = ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString();
            var cfg = new NHibernate.Cfg.Configuration();
            cfg.DataBaseIntegration(x =>
            {
                x.ConnectionString = conn;
                x.Driver<SqlClientDriver>();
                x.Dialect<MsSql2012Dialect>();
            });
            cfg.AddAssembly(Assembly.GetAssembly(typeof(ResourceAccess.NHibernateDB.ContentItems_Published)));
            var sessionFactory = cfg.BuildSessionFactory();
            using (var session = sessionFactory.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var items = session.CreateCriteria<ResourceAccess.NHibernateDB.ContentItems_Published>().List<ResourceAccess.NHibernateDB.ContentItems_Published>();
                    tx.Commit();
                    if (items == null || !items.Any())
                        throw new Exception("No Rows Returned");
                }
            }
        }

        #endregion Session

        #endregion NHibernate

        #region Dapper

        private void Dapper(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.SingleTable:
                    RequestResponse_Dapper_Single_Table(input);
                    break;
                case DBCallType.Join:
                    RequestResponse_Dapper_Join(input);
                    break;
                case DBCallType.Insert:
                    RequestResponse_Dapper_Insert(input);
                    break;
                case DBCallType.Complex:
                    RequestResponse_Dapper_Complex(input);
                    break;
                case DBCallType.DoubleCall:
                    RequestResponse_Dapper_DoubleCall(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_Dapper_Single_Table(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());
            var items = (conn.Query<ResourceAccess.Dapper.ContentItems_Published>(@"SELECT * FROM[dbo].[ContentItems_Published]")).ToList();
            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }

        public void RequestResponse_Dapper_Join(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());

            var items = (conn.Query<ResourceAccess.Dapper.ContentItems_Published__ContentItems_LocalizedItem>(@"SELECT cli.*, cip.* FROM
                                                    [DBO].[ContentItems_LocalizedItem] cli INNER JOIN [DBO].[ContentItems_Published] cip
                                                    ON cli.LocalizedItemID = cip.LocalizedItemID")).ToList();
            if (items == null || !items.Any())
                throw new Exception("No Rows Returned");
        }

        public void RequestResponse_Dapper_Insert(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);
            var sql = @"INSERT INTO [dbo].[Dump] ([text]) VALUES (@Text); SELECT CAST(SCOPE_IDENTITY() as int)";

            ResourceAccess.Dapper.Dump dump = new ResourceAccess.Dapper.Dump()
            {
                text = input.TestName
            };

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());

            int id = (conn.Query<int>(sql,dump)).Single();
            id.Should().BeGreaterOrEqualTo(1);
        }

        public void RequestResponse_Dapper_Complex(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            DataTable tableParameter = new DataTable();
            tableParameter.Columns.Add("n");
            tableParameter.Rows.Add(142);

            ResourceAccess.Dapper.Dump dump = new ResourceAccess.Dapper.Dump()
            {
                text = input.TestName
            };

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());

            var results = (conn.Query<ResourceAccess.Dapper.AbiLibraryItemData>(ComplexSQL, new { PublishedItemMetadataIds = tableParameter.AsTableValuedParameter("Ids") })).ToList(); ;
            results.Should().NotBeNull();
        }

        public void RequestResponse_Dapper_DoubleCall(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());

            var items = (conn.Query<ResourceAccess.Dapper.ContentItems_Published__ContentItems_LocalizedItem>(@"SELECT cli.*, cip.* FROM
                                                    [DBO].[ContentItems_LocalizedItem] cli INNER JOIN [DBO].[ContentItems_Published] cip
                                                    ON cli.LocalizedItemID = cip.LocalizedItemID")).ToList();
            var items2 = (conn.Query<ResourceAccess.Dapper.ContentItems_Published__ContentItems_LocalizedItem>(@"SELECT s.*, l.* FROM 
                                                    [DBO].[ContentItems_LocalizedShortcut] s inner join [dbo].[ContentItems_LocalizedItem] l
                                                    ON s.LocalizedItemID = l.LocalizedItemID")).ToList();
            items.Should().NotBeNull();
            items2.Should().NotBeNull();
        }

        #endregion Dapper

        #region Mixed Calls

        private void MixedCalls(DBRequestDataContract input)
        {
            switch (input.DBCallType)
            {
                case DBCallType.DapperEF:
                    RequestResponse_MixedCalls_DapperEF(input);
                    break;
                case DBCallType.EFDapper:
                    RequestResponse_MixedCalls_DapperEF(input);
                    break;
                default:
                    throw new Exception("Unknown Call Type");
            }
        }

        public void RequestResponse_MixedCalls_DapperEF(RequestDataContract input)
        {
            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());

            var items = (conn.Query<ResourceAccess.Dapper.ContentItems_Published__ContentItems_LocalizedItem>(@"SELECT cli.*, cip.* FROM
                                                [DBO].[ContentItems_LocalizedItem] cli INNER JOIN [DBO].[ContentItems_Published] cip
                                                ON cli.LocalizedItemID = cip.LocalizedItemID")).ToList();

            using (VimagoEntities context = new VimagoEntities())
            {
                var items2 = (from l in context.ContentItems_LocalizedShortcut
                              join c in context.ContentItems_LocalizedItem on l.LocalizedItemID equals c.LocalizedItemID
                              select new
                              {
                                  l.LocalizedItemID,
                                  l.LocalizedTargetId,
                                  l.LocalizedTargetTypeID,
                                  c.CreationDate,
                                  c.CreatorID,
                                  c.Description,
                                  c.HasPicture,
                                  c.IsLockedBy,
                                  c.ItemID,
                                  c.LanguageID,
                                  c.LastModificationBy,
                                  c.LastModificationDate,
                                  c.LocalizationStrategy,
                                  c.LocalizedType,
                                  c.Name,
                                  c.PublishState,
                                  c.UniqueIdentifier

                              }).ToList();
                //items.Should().NotBeNull();
                items2.Should().NotBeNull();
            }
        }

        public void RequestResponse_MixedCalls_EFDapper(RequestDataContract input)
        {

            DateTime serverReceived = DateTime.Now;
            ServiceHelper.Wait(input);

            IDbConnection conn = new SqlConnection(ConfigurationManager.AppSettings["ConnectionString.SQL Server (SqlClient)"].ToString());

            using (VimagoEntities context = new VimagoEntities())
            {
                var items2 = (from l in context.ContentItems_LocalizedShortcut
                              join c in context.ContentItems_LocalizedItem on l.LocalizedItemID equals c.LocalizedItemID
                              select new
                              {
                                  l.LocalizedItemID,
                                  l.LocalizedTargetId,
                                  l.LocalizedTargetTypeID,
                                  c.CreationDate,
                                  c.CreatorID,
                                  c.Description,
                                  c.HasPicture,
                                  c.IsLockedBy,
                                  c.ItemID,
                                  c.LanguageID,
                                  c.LastModificationBy,
                                  c.LastModificationDate,
                                  c.LocalizationStrategy,
                                  c.LocalizedType,
                                  c.Name,
                                  c.PublishState,
                                  c.UniqueIdentifier

                              }).ToList();
                //items.Should().NotBeNull();
                items2.Should().NotBeNull();
            }

            var items = (conn.Query<ResourceAccess.Dapper.ContentItems_Published__ContentItems_LocalizedItem>(@"SELECT cli.*, cip.* FROM
                                                [DBO].[ContentItems_LocalizedItem] cli INNER JOIN [DBO].[ContentItems_Published] cip
                                                ON cli.LocalizedItemID = cip.LocalizedItemID")).ToList();

            
        }

        #endregion Mixed Calls

        #endregion
    }
}